package cn.com.ty.lift.business.framework.conf;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.JsonTokenId;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;

import java.io.IOException;
import java.math.BigDecimal;

/**
 * BigDecimal反序列化
 *
 * @author wcz
 * @since 2020/4/26
 */
public class BigDecimalDeserializer extends StdScalarDeserializer<BigDecimal> {

    public static final BigDecimalDeserializer instance = new BigDecimalDeserializer();

    public BigDecimalDeserializer() {
        super(BigDecimal.class);
    }

    @Override
    public BigDecimal deserialize(JsonParser p, DeserializationContext ctxt) throws IOException, JsonProcessingException {
        switch (p.getCurrentTokenId()) {
            case JsonTokenId.ID_NUMBER_INT:
            case JsonTokenId.ID_NUMBER_FLOAT:
                return p.getDecimalValue().setScale(2, BigDecimal.ROUND_HALF_UP);
            case JsonTokenId.ID_STRING:
                String text = p.getText().trim();
                // note: no need to call `coerce` as this is never primitive
                if (_isEmptyOrTextualNull(text)) {
                    _verifyNullForScalarCoercion(ctxt, text);
                    return getNullValue(ctxt).setScale(2, BigDecimal.ROUND_HALF_UP);
                }
                _verifyStringForScalarCoercion(ctxt, text);
                try {
                    return new BigDecimal(text).setScale(2, BigDecimal.ROUND_HALF_UP);
                } catch (IllegalArgumentException iae) {
                }
                return ((BigDecimal) ctxt.handleWeirdStringValue(_valueClass, text,
                        "not a valid representation")).setScale(2, BigDecimal.ROUND_HALF_UP);
            case JsonTokenId.ID_START_ARRAY:
                return _deserializeFromArray(p, ctxt).setScale(2, BigDecimal.ROUND_HALF_UP);
        }
        // Otherwise, no can do:
        return ((BigDecimal) ctxt.handleUnexpectedToken(_valueClass, p)).setScale(2, BigDecimal.ROUND_HALF_UP);
    }
}
